let
  
  mergeSets = defaults: parts:
    builtins.foldl'
      (a: b: a // b)
      {}
      (builtins.map (f: f defaults) parts);

  setToFilePaths = ordering: parts:
    let

      grabPaths = item: path:
        if (builtins.isString item) then
          ["${builtins.concatStringsSep "/" path}/${item}.png"]
        else
          builtins.concatMap
            (p: grabPaths item.${p} (path ++ [p]))
          (builtins.attrNames item);

      initial = spec: order:
        builtins.concatMap
          (o: if builtins.hasAttr o spec then
                grabPaths spec.${o} [o]
              else [])
        order;
        
        
    in
      initial parts ordering;

  mkLibPicrew = { pkgs, ... }@args: rec {
    renderPicrew = mkRenderPicrew args;
    renderPicrews = name: defs: specs:
      pkgs.buildEnv {
        inherit name;
        paths = 
          builtins.map (spec: renderPicrew {
            defaults = defs;
            inherit (spec) name parts;
          }) specs;
      };
  };

  mkRenderPicrew = {
    images,
    ordering,
    pkgs
  }@args: {
    name,
    parts,
    defaults ? {},
    src ? images,
    ordering ? args.ordering
  }: let

      compiledSpec = mergeSets defaults parts;
      argList = setToFilePaths ordering compiledSpec;
      imList =
        if builtins.length argList <= 2 then
          builtins.concatStringsSep " " argList
        else
          builtins.head argList + " "
            + builtins.concatStringsSep " -composite "
              (builtins.tail argList);
     
    in pkgs.stdenv.mkDerivation {
      inherit name src;
      buildInputs = [ pkgs.imagemagick ];
      installPhase = ''
        mkdir -p $out
        convert ${imList} -composite $out/${name}.png
      '';
    };

in {
  inherit mkLibPicrew;
}
